
#include <fstream>
#include <iostream>

#include "alphabet.h"
#include "pmf.h"
#include "adec.h"
#include "decode.h"


bool decode( const ACfg& acfg, std::string& bitstream, std::string& output )
{
  //===== set alphabet and pmf =====
  Alphabet* pAlphabet = 0;
  Pmf*      pPmf      = 0;
  switch( acfg.version )
  {
  case 0:
    pAlphabet = new AlpbabetABCD        ();
    pPmf      = new Pmf4Fixed           ( acfg.Vbits );
    break;
  case 1:
    pAlphabet = new Alphabet256         ();
    pPmf      = new Pmf256Fixed         ( acfg.Vbits );
    break;
  default:
    assert(false);
    break;
  }
  Alphabet& alphabet  = *pAlphabet;
  Pmf&      pmf       = *pPmf;

  //===== open output file =====
  std::ofstream outfile  ( output, std::ios::binary );
  if( !outfile.is_open() )
  {
    std::cerr << "\nERROR: Could not open output file \"" << output << "\"\n";
    return true;
  }

  //===== init arithmetic decoder =====
  ADec adec( acfg, bitstream );
  if( !adec.isValid() )
  {
    std::cerr << "\nERROR: Could not open bitstream file \"" << bitstream << "\"\n";
    return true;
  }

  //===== decode symbols =====
  while( adec.moreSymbols() )
  {
    // decode
    uint8_t index = adec.decode( pmf );
    pmf.update( index );

    // write symbol
    char symbol = char( alphabet.getSymbol( index ) );
    outfile.put( symbol );
  }
  outfile.close();

  //===== delete ======
  delete pAlphabet;
  delete pPmf;

  //===== output and return =====
  std::cout << "\ndone: " << adec.getNumDecoded() << " symbols decoded.\n\n";
  return false;
}

