
#include "obitstream.h"
#include <cassert>


OBitstream::OBitstream( std::string& filename )
  : file        ( filename, std::ios::binary )
  , valid       ( file.is_open() )
  , nextBits    ( 0 )
  , numNextBits ( 0 )
{
  // reserve first four bytes for coding number of symbols
  if( valid )
  {
    for( int k = 0; k < 4; k++ )
    {
      file.put( 0 );
    }
  }
}


OBitstream::~OBitstream()
{
  assert( !valid );
}


void OBitstream::addBit( uint8_t nextBit )
{
  assert( valid       );
  assert( nextBit < 2 );
  nextBits = ( nextBits << 1 ) + uint8_t( nextBit );
  if( ++numNextBits == 8 )
  {
    file.put( char( nextBits ) );
    nextBits    = 0;
    numNextBits = 0;
  }
}


void OBitstream::terminate( uint32_t numSymbols )
{
  assert( valid );
  
  // fill last byte
  while( numNextBits )
  {
    addBit( 0 );
  }

  // code number of symbols at start of bitstream
  file.seekp( std::streampos(0) );
  for( int k = 0; k < 4; k++ )
  {
    file.put( numSymbols & 0xff );  
    numSymbols >>= 8;
  }

  // close file and set bitstream to invalid
  file.close();
  valid = false;
}

